#include "polynome.hpp"

//*****************
//* Constructeurs *
//*****************

//a) Cree le polynome 0
Polynome::Polynome(){
  taille=0;
  coeffs=nullptr;
}

//b) Cree le polynome X^n
Polynome::Polynome(size_t n){
  taille=n+1;
  coeffs=new double[n+1]; //cree un tableau de taille n+1
  for(size_t i=0;i<n;++i){
    coeffs[i]=0;
  }
  coeffs[n]=1;
}


Polynome::Polynome(initializer_list<double> l){
  if(l.size()==0){
    //Pas de coefficients
    taille=0;
    coeffs=nullptr;
  }
  else{
    taille=l.size();
    coeffs=new double[taille];
    int i=taille-1;
    for(initializer_list<double>::iterator it=l.begin();it!=l.end();++it){
      coeffs[i]=*it;
      --i;
    }
  }
}

//c) Destructeur 
Polynome::~Polynome(){
  if(coeffs!=nullptr){
    delete[] coeffs;
  }
}

//d) Degree
int Polynome::degre() const{
  if(coeffs==nullptr){ //Le polynome courant est nul
    return -1;
  }
  return taille-1;
}

//e) Acces aux coefficients
double& Polynome::operator[](size_t i){
  assert(coeffs!=nullptr);
  assert(i<taille);
  return coeffs[i];
}

double Polynome::operator[](size_t i) const{
  assert(coeffs!=nullptr);
  assert(i<taille);
  return coeffs[i];
}

//f) Operator d'affectation=

Polynome& Polynome::operator=(const Polynome& Q){
  //Si le polynome courant est non nul
  //on désaloue son tableau de coefficients
  if(coeffs!=nullptr){
    delete[] coeffs;
  }
  taille=Q.taille;
  //Cas où Q est nulle
  if(Q.coeffs==nullptr){
    coeffs=nullptr;
    return *this;
  }
  //Sinon
  coeffs=new double[taille];
  for(int i=0;i<taille;++i){
    coeffs[i]=Q.coeffs[i];
  }
  return *this;
}


//Operator<<
ostream& operator<<(ostream& os,const Polynome& P){
  if(P.degre()==-1){
    os<<"0";
    return os;
  }
  for(int i=P.degre();i>=0;--i){
    double c=P[i];
    
    if(c>0 and i!=P.degre()) os<<" + ";
    if(c>0) os<<c;

    if(c<0) os<<" - "<<-c;
    if(c!=0){
      if(i>=1){
	os<<" X";
      }
      if(i>=2){
	os<<'^'<<i;
      }
    }
  }
}
